using System;
using System.Collections.ObjectModel;
using ThinkGeo.MapSuite.Core;
using System.Collections;

namespace CSSamples.Samples
{
    public class TitleWrappedLegendAdornmentLayer : LegendAdornmentLayer
    {
        private int rowHeight; // height for each item
        private int legendItems; // # of items to show

        public TitleWrappedLegendAdornmentLayer(int itemHeight, int items)
        {
            rowHeight = itemHeight;
            legendItems = items;
        }

        protected override void DrawCore(GeoCanvas canvas, Collection<SimpleCandidate> labelsInAllLayers)
        {
            // find title text/style/font -> wrapped and adjust height?
            WrapTitleWithHeight(canvas, this.Title);

            // decide layer height
            this.Height = legendItems * rowHeight + this.Title.Height;

            base.DrawCore(canvas, labelsInAllLayers);
        }

        protected void WrapTitleWithHeight(GeoCanvas adornmentGeoCanvas, LegendItem titleItem)
        {
            string text = titleItem.TextStyle.TextColumnName.Replace("\r\n", " ").Replace("\t", " ").Trim();

            DrawingRectangleF drawingRectangleF = adornmentGeoCanvas.MeasureText(text, titleItem.TextStyle.Font);

            //Compares the bounding box in screen coordinates with the size of the text.
            if (this.Title.Width < drawingRectangleF.Width)
            {
                //If not, applies the word wrapping.

                string[] Words = Wrap(adornmentGeoCanvas, text, titleItem.TextStyle.Font, (int)this.Title.Width); //drawingRectangleF.Width

                string newLineText = "";

                for (int i = 0; i < Words.Length; i++)
                {
                    newLineText = newLineText + Words[i] + Environment.NewLine;
                }

                titleItem.TextStyle.AllowLineCarriage = true;
                titleItem.TextStyle.TextColumnName = newLineText;

                drawingRectangleF = adornmentGeoCanvas.MeasureText(newLineText, titleItem.TextStyle.Font);
                // adjust height
                if (titleItem.Height < drawingRectangleF.Height)
                {
                    titleItem.Height = drawingRectangleF.Height + 5;
                }
            }

            return;
        }

        //Function to get the maximum length of the string without having a new line.
        private int GetMaxStringLength(GeoCanvas canvas, float currentWidth)
        {
            //For now, the value is   You can write your own logic to calculate that value based on
            // the size of the text and the bounding box to hold it.

            int result = (int)currentWidth; // 10;

            return result;
        }

        //Function taken from http://www.velocityreviews.com/forums/t20370-word-wrap-line-break-code-and-algorithm-for-c.html

        private string[] Wrap(GeoCanvas theCanvas, string text, GeoFont txtFont, int maxLength)
        {
            text = text.Replace("\n", " ");

            text = text.Replace("\r", " ");

            text = text.Replace(".", ". ");

            text = text.Replace(">", "> ");

            text = text.Replace("\t", " ");

            text = text.Replace(",", ", ");

            text = text.Replace(";", "; ");

            text = text.Replace("<br>", " ");

            text = text.Replace(" ", " ");

            string[] Words = text.Split(' ');

            int currentLineLength = 0;
            float wordLength = 0;
            ArrayList Lines = new ArrayList(text.Length / maxLength);

            string currentLine = "";

            bool InTag = false;

            foreach (string currentWord in Words)
            {
                //ignore html

                if (currentWord.Length > 0)
                {

                    if (currentWord.Substring(0, 1) == "<")

                        InTag = true;

                    if (InTag)
                    {
                        //handle filenames inside html tags

                        if (currentLine.EndsWith("."))
                        {
                            currentLine += currentWord;
                        }
                        else

                            currentLine += " " + currentWord;

                        if (currentWord.IndexOf(">") > -1)

                            InTag = false;
                    }
                    else
                    {
                        wordLength = theCanvas.MeasureText(currentWord, txtFont).Width;

                        // if (currentLineLength + currentWord.Length + 1 < maxLength)
                        if (currentLineLength + wordLength + 1 < maxLength)
                        {
                            currentLine += " " + currentWord;

                            // currentLineLength += (currentWord.Length + 1);
                            currentLineLength += ((int)wordLength + 1);
                        }
                        else
                        {
                            Lines.Add(currentLine);

                            currentLine = currentWord;

                            currentLineLength = (int)wordLength; // currentWord.Length;
                        }
                    }
                }

            }

            if (currentLine != "")

                Lines.Add(currentLine);

            string[] textLinesStr = new string[Lines.Count];

            Lines.CopyTo(textLinesStr, 0);

            return textLinesStr;
        }// function
    }// class

    public partial class DisplayASimpleMap : System.Web.UI.Page
    {
        protected void Page_Load(object sender, EventArgs e)
        {
            if (!Page.IsPostBack)
            {
                TitleWrappedLegendAdornmentLayer legend = new TitleWrappedLegendAdornmentLayer(30, 3);
                legend.Title = new LegendItem();
                legend.Title.TextStyle = TextStyles.Capital1("Hi This is a Test Title");

                LegendItem item1 = new LegendItem();
                item1.TextStyle = TextStyles.City4("My name is item1 and I came from Canada");
                LegendItem item2 = new LegendItem();
                item2.TextStyle = TextStyles.City4("My name is item2 and I came from Austrlia");

                legend.LegendItems.Add(item1);
                legend.LegendItems.Add(item2);

                Map1.AdornmentOverlay.Layers.Add(legend);

                Map1.MapBackground.BackgroundBrush = new GeoSolidBrush(GeoColor.FromHtml("#E5E3DF"));
                Map1.CurrentExtent = new RectangleShape(-2000000, 2000000, 2000000, -2000000);
                Map1.MapUnit = GeographyUnit.Meter;
            }
        }
    }
}